@@ -1,10 +1,9 @@
-from typing import Any, Dict, List, Mapping, Optional, Sequence, Set, TypeVar, cast
+from collections.abc import Mapping, Sequence
+from typing import Any, Optional, TypeVar, cast
 
 import dagster._check as check
-from dagster._utils import ensure_single_item
-
-from .config_type import ConfigScalarKind, ConfigType, ConfigTypeKind
-from .errors import (
+from dagster._config.config_type import ConfigScalarKind, ConfigType, ConfigTypeKind
+from dagster._config.errors import (
     EvaluationError,
     create_array_error,
     create_dict_type_mismatch_error,
@@ -17,18 +16,20 @@ from .errors import (
     create_missing_required_field_error,
     create_missing_required_fields_error,
     create_none_not_allowed_error,
+    create_pydantic_env_var_error,
     create_scalar_error,
     create_selector_multiple_fields_error,
     create_selector_multiple_fields_no_field_selected_error,
     create_selector_type_error,
     create_selector_unspecified_value_error,
 )
-from .evaluate_value_result import EvaluateValueResult
-from .field import resolve_to_config_type
-from .post_process import post_process_config
-from .snap import ConfigFieldSnap, ConfigSchemaSnapshot, ConfigTypeSnap
-from .stack import EvaluationStack
-from .traversal_context import ValidationContext
+from dagster._config.evaluate_value_result import EvaluateValueResult
+from dagster._config.field import resolve_to_config_type
+from dagster._config.post_process import post_process_config
+from dagster._config.snap import ConfigFieldSnap, ConfigSchemaSnapshot, ConfigTypeSnap
+from dagster._config.stack import EvaluationStackRoot
+from dagster._config.traversal_context import ValidationContext
+from dagster._utils import ensure_single_item
 
 VALID_FLOAT_TYPES = tuple([int, float])
 
@@ -54,11 +55,10 @@ def is_config_scalar_valid(config_type_snap: ConfigTypeSnap, config_value: objec
 
 
 def validate_config(config_schema: object, config_value: T) -> EvaluateValueResult[T]:
-    config_type = resolve_to_config_type(config_schema)
-    config_type = check.inst(cast(ConfigType, config_type), ConfigType)
+    config_type = check.inst(resolve_to_config_type(config_schema), ConfigType)
 
     return validate_config_from_snap(
-        config_schema_snapshot=config_type.get_schema_snapshot(),
+        config_schema_snapshot=config_type.schema_snapshot,
         config_type_key=config_type.key,
         config_value=config_value,
     )
@@ -73,13 +73,15 @@ def validate_config_from_snap(
         ValidationContext(
             config_schema_snapshot=config_schema_snapshot,
             config_type_snap=config_schema_snapshot.get_config_snap(config_type_key),
-            stack=EvaluationStack(entries=[]),
+            stack=EvaluationStackRoot(),
         ),
         config_value,
     )
 
 
 def _validate_config(context: ValidationContext, config_value: object) -> EvaluateValueResult[Any]:
+    from dagster._config.field_utils import EnvVar, IntEnvVar
+
     check.inst_param(context, "context", ValidationContext)
 
     kind = context.config_type_snap.kind
@@ -100,6 +102,13 @@ def _validate_config(context: ValidationContext, config_value: object) -> Evalua
     if kind == ConfigTypeKind.SCALAR:
         if not is_config_scalar_valid(context.config_type_snap, config_value):
             return EvaluateValueResult.for_error(create_scalar_error(context, config_value))
+        # If user passes an EnvVar or IntEnvVar to a non-structured run config dictionary, throw explicit error
+        if context.config_type_snap.scalar_kind == ConfigScalarKind.STRING and isinstance(
+            config_value, (EnvVar, IntEnvVar)
+        ):
+            return EvaluateValueResult.for_error(
+                create_pydantic_env_var_error(context, config_value)
+            )
         return EvaluateValueResult.for_value(config_value)
     elif kind == ConfigTypeKind.SELECTOR:
         return validate_selector_config(context, config_value)
@@ -129,7 +138,7 @@ def _validate_scalar_union_config(
     if isinstance(config_value, (dict, list)):
         return _validate_config(
             context.for_new_config_type_key(context.config_type_snap.non_scalar_type_key),
-            cast(T, config_value),
+            cast("T", config_value),
         )
     else:
         return _validate_config(
@@ -152,7 +161,7 @@ def _validate_empty_selector_config(
     if defined_field_snap.is_required:
         return EvaluateValueResult.for_error(create_selector_unspecified_value_error(context))
 
-    return EvaluateValueResult.for_value(cast(Mapping[str, object], {}))
+    return EvaluateValueResult.for_value(cast("Mapping[str, object]", {}))
 
 
 def validate_selector_config(
@@ -170,12 +179,12 @@ def validate_selector_config(
     if config_value == {}:
         return _validate_empty_selector_config(context)
 
-    # Now we ensure that the used-provided config has only a a single entry
+    # Now we ensure that the used-provided config has only a single entry
     # and then continue the validation pass
 
     if not isinstance(config_value, dict):
         return EvaluateValueResult.for_error(create_selector_type_error(context, config_value))
-    config_value = cast(Mapping[str, object], config_value)
+    config_value = cast("Mapping[str, object]", config_value)
 
     if len(config_value) > 1:
         return EvaluateValueResult.for_error(
@@ -191,27 +200,27 @@ def validate_selector_config(
 
     child_evaluate_value_result = _validate_config(
         context.for_field_snap(field_snap),
-        # This is a very particular special case where we want someone
-        # to be able to select a selector key *without* a value
-        #
-        # e.g.
-        # storage:
-        #   filesystem:
-        #
-        # And we want the default values of the child elements of filesystem:
-        # to "fill in"
-        {}
-        if field_value is None
-        and ConfigTypeKind.has_fields(
-            context.config_schema_snapshot.get_config_snap(field_snap.type_key).kind
-        )
-        else field_value,
+        (
+            # This is a very particular special case where we want someone
+            # to be able to select a selector key *without* a value
+            #
+            # e.g.
+            # storage:
+            #   filesystem:
+            #
+            # And we want the default values of the child elements of filesystem:
+            # to "fill in"
+            {}
+            if field_value is None
+            and ConfigTypeKind.has_fields(
+                context.config_schema_snapshot.get_config_snap(field_snap.type_key).kind
+            )
+            else field_value
+        ),
     )
 
     if child_evaluate_value_result.success:
-        return EvaluateValueResult.for_value(  # type: ignore
-            {field_name: child_evaluate_value_result.value}
-        )
+        return EvaluateValueResult.for_value({field_name: child_evaluate_value_result.value})
     else:
         return child_evaluate_value_result
 
@@ -224,7 +233,7 @@ def _validate_shape_config(
     check.bool_param(check_for_extra_incoming_fields, "check_for_extra_incoming_fields")
 
     field_aliases = check.opt_dict_param(
-        cast(Dict[str, str], context.config_type_snap.field_aliases),
+        cast("dict[str, str]", context.config_type_snap.field_aliases),
         "field_aliases",
         key_type=str,
         value_type=str,
@@ -232,15 +241,15 @@ def _validate_shape_config(
 
     if not isinstance(config_value, dict):
         return EvaluateValueResult.for_error(create_dict_type_mismatch_error(context, config_value))
-    config_value = cast(Dict[str, object], config_value)
+    config_value = cast("dict[str, object]", config_value)
 
     field_snaps = check.not_none(context.config_type_snap.fields)
-    defined_field_names = {cast(str, fs.name) for fs in field_snaps}
+    defined_field_names = cast("set[str]", {fs.name for fs in field_snaps})
     defined_field_names = defined_field_names.union(set(field_aliases.values()))
 
     incoming_field_names = set(config_value.keys())
 
-    errors: List[EvaluationError] = []
+    errors: list[EvaluationError] = []
 
     if check_for_extra_incoming_fields:
         _append_if_error(
@@ -289,7 +298,7 @@ def _validate_shape_config(
     if errors:
         return EvaluateValueResult.for_errors(errors)
     else:
-        return EvaluateValueResult.for_value(config_value)  # type: ignore
+        return EvaluateValueResult.for_value(config_value)
 
 
 def validate_permissive_shape_config(
@@ -311,7 +320,7 @@ def validate_map_config(
 
     if not isinstance(config_value, dict):
         return EvaluateValueResult.for_error(create_map_error(context, config_value))
-    config_value = cast(Dict[object, object], config_value)
+    config_value = cast("dict[object, object]", config_value)
 
     evaluation_results = [
         _validate_config(context.for_map_key(key), key) for key in config_value.keys()
@@ -323,9 +332,13 @@ def validate_map_config(
     errors = []
     for result in evaluation_results:
         if not result.success:
-            errors += cast(List, result.errors)
+            errors += cast("list", result.errors)
 
-    return EvaluateValueResult(not bool(errors), config_value, errors)
+    return EvaluateValueResult(
+        success=not bool(errors),
+        value=config_value,
+        errors=errors,
+    )
 
 
 def validate_shape_config(
@@ -338,13 +351,13 @@ def validate_shape_config(
     return _validate_shape_config(context, config_value, check_for_extra_incoming_fields=True)
 
 
-def _append_if_error(errors: List[EvaluationError], maybe_error: Optional[EvaluationError]) -> None:
+def _append_if_error(errors: list[EvaluationError], maybe_error: Optional[EvaluationError]) -> None:
     if maybe_error:
         errors.append(maybe_error)
 
 
 def _check_for_extra_incoming_fields(
-    context: ValidationContext, defined_field_names: Set[str], incoming_field_names: Set[str]
+    context: ValidationContext, defined_field_names: set[str], incoming_field_names: set[str]
 ) -> Optional[EvaluationError]:
     extra_fields = list(incoming_field_names - defined_field_names)
 
@@ -359,16 +372,16 @@ def _check_for_extra_incoming_fields(
 def _compute_missing_fields_error(
     context: ValidationContext,
     field_snaps: Sequence[ConfigFieldSnap],
-    incoming_fields: Set[str],
+    incoming_fields: set[str],
     field_aliases: Mapping[str, str],
 ) -> Optional[EvaluationError]:
-    missing_fields: List[str] = []
+    missing_fields: list[str] = []
 
     for field_snap in field_snaps:
-        field_alias = field_aliases.get(cast(str, field_snap.name))
+        field_alias = field_aliases.get(cast("str", field_snap.name))
         if field_snap.is_required and field_snap.name not in incoming_fields:
             if field_alias is None or field_alias not in incoming_fields:
-                missing_fields.append(cast(str, field_snap.name))
+                missing_fields.append(cast("str", field_snap.name))
 
     if missing_fields:
         if len(missing_fields) == 1:
@@ -390,18 +403,22 @@ def validate_array_config(
 
     evaluation_results = [
         _validate_config(context.for_array(index), config_item)
-        for index, config_item in enumerate(cast(List[object], config_value))
+        for index, config_item in enumerate(cast("list[object]", config_value))
     ]
 
-    values: List[object] = []
-    errors: List[EvaluationError] = []
+    values: list[object] = []
+    errors: list[EvaluationError] = []
     for result in evaluation_results:
         if result.success:
             values.append(result.value)
         else:
             errors.extend(check.not_none(result.errors))
 
-    return EvaluateValueResult(not bool(errors), values, errors)
+    return EvaluateValueResult(
+        success=not bool(errors),
+        value=values,
+        errors=errors,
+    )
 
 
 def validate_enum_config(
@@ -422,9 +439,9 @@ def validate_enum_config(
 
 def process_config(
     config_type: object, config_dict: Mapping[str, object]
-) -> EvaluateValueResult[Mapping[str, object]]:
+) -> EvaluateValueResult[Mapping[str, Any]]:
     config_type = resolve_to_config_type(config_type)
-    config_type = check.inst(cast(ConfigType, config_type), ConfigType)
+    config_type = check.inst(cast("ConfigType", config_type), ConfigType)
     validate_evr = validate_config(config_type, config_dict)
     if not validate_evr.success:
         return validate_evr
