@@ -129,6 +129,10 @@ class AssetGraphView(LoadingContext):
             dynamic_partitions_store=self._queryer, temporal_context=temporal_context
         )
 
+    @property
+    def partition_loading_context(self) -> PartitionLoadingContext:
+        return self._partition_loading_context
+
     @property
     def instance(self) -> "DagsterInstance":
         return self._instance
