@@ -1,4 +1,4 @@
-from .config import (
+from dagster._core.container_context.config import (
     SHARED_CONTAINER_CONTEXT_SCHEMA as SHARED_CONTAINER_CONTEXT_SCHEMA,
     process_shared_container_context_config as process_shared_container_context_config,
 )
