@@ -71,11 +71,7 @@ class MinimalAssetMaterializationHealthState(LoadableBy[AssetKey]):
         asset_materialization_health_states = (
             context.instance.get_minimal_asset_materialization_health_state_for_assets(list(keys))
         )
-
-        if asset_materialization_health_states is None:
-            return [None for _ in keys]
-        else:
-            return [asset_materialization_health_states.get(key) for key in keys]
+        return [asset_materialization_health_states.get(key) for key in keys]
 
 
 @whitelist_for_serdes
@@ -231,11 +227,7 @@ class AssetMaterializationHealthState(LoadableBy[AssetKey]):
         asset_materialization_health_states = (
             context.instance.get_asset_materialization_health_state_for_assets(list(keys))
         )
-
-        if asset_materialization_health_states is None:
-            return [None for _ in keys]
-        else:
-            return [asset_materialization_health_states.get(key) for key in keys]
+        return [asset_materialization_health_states.get(key) for key in keys]
 
 
 async def _get_is_currently_failed_and_latest_terminal_run_id(
