@@ -642,11 +642,13 @@ class DependencyDefinition(
 
     .. code-block:: python
 
+        from dagster import DependencyDefinition
+
         dependency_structure = {
             'my_downstream_op': {
                 'input': DependencyDefinition('my_upstream_op', 'result')
-            }
-            'my_downstream_op': {
+            },
+            'my_other_downstream_op': {
                 'input': DependencyDefinition('my_upstream_graph', 'result')
             }
         }
@@ -657,6 +659,8 @@ class DependencyDefinition(
 
     .. code-block:: python
 
+        from dagster import job
+
         @job
         def the_job():
             node_b(node_a())
