@@ -485,10 +485,10 @@ def build_reconstructable_job(
             from dagster import JobDefinition, job, build_reconstructable_job
 
             class JobFactory:
-                def make_job(*args, **kwargs):
+                def make_job(self, *args, **kwargs):
 
                     @job
-                    def _job(...):
+                    def _job():
                         ...
 
                     return _job
@@ -499,9 +499,9 @@ def build_reconstructable_job(
 
             factory = JobFactory()
 
-            foo_job_args = (...,...)
+            foo_job_args = (..., ...)
 
-            foo_job_kwargs = {...:...}
+            foo_job_kwargs = {...}
 
             foo_job = factory.make_job(*foo_job_args, **foo_job_kwargs)
 
