@@ -1,7 +1,10 @@
-from typing import Mapping, NamedTuple, Optional, Sequence
+from collections.abc import Mapping, Sequence
+from typing import AbstractSet, NamedTuple, Optional  # noqa: UP035
 
 import dagster._check as check
 from dagster._core.definitions import NodeHandle
+from dagster._core.definitions.asset_key import EntityKey
+from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.repository_definition import RepositoryLoadData
 from dagster._core.execution.plan.inputs import (
     StepInput,
@@ -33,14 +36,17 @@ def create_execution_plan_snapshot_id(execution_plan_snapshot: "ExecutionPlanSna
     return create_snapshot_id(execution_plan_snapshot)
 
 
-@whitelist_for_serdes(skip_when_empty_fields={"repository_load_data"})
+@whitelist_for_serdes(
+    storage_field_names={"job_snapshot_id": "pipeline_snapshot_id"},
+    skip_when_empty_fields={"repository_load_data"},
+)
 class ExecutionPlanSnapshot(
     NamedTuple(
         "_ExecutionPlanSnapshot",
         [
             ("steps", Sequence["ExecutionStepSnap"]),
             ("artifacts_persisted", bool),
-            ("pipeline_snapshot_id", str),
+            ("job_snapshot_id", str),
             ("step_keys_to_execute", Sequence[str]),
             ("initial_known_state", Optional[KnownExecutionState]),
             ("snapshot_version", Optional[int]),
@@ -58,22 +64,23 @@ class ExecutionPlanSnapshot(
     # added step_output_versions
     # removed step_output_versions
     # added repository_load_data
+
     def __new__(
         cls,
         steps: Sequence["ExecutionStepSnap"],
         artifacts_persisted: bool,
-        pipeline_snapshot_id: str,
+        job_snapshot_id: str,
         step_keys_to_execute: Optional[Sequence[str]] = None,
         initial_known_state: Optional[KnownExecutionState] = None,
         snapshot_version: Optional[int] = None,
         executor_name: Optional[str] = None,
         repository_load_data: Optional[RepositoryLoadData] = None,
     ):
-        return super(ExecutionPlanSnapshot, cls).__new__(
+        return super().__new__(
             cls,
             steps=check.sequence_param(steps, "steps", of_type=ExecutionStepSnap),
             artifacts_persisted=check.bool_param(artifacts_persisted, "artifacts_persisted"),
-            pipeline_snapshot_id=check.str_param(pipeline_snapshot_id, "pipeline_snapshot_id"),
+            job_snapshot_id=check.str_param(job_snapshot_id, "job_snapshot_id"),
             step_keys_to_execute=check.opt_sequence_param(
                 step_keys_to_execute, "step_keys_to_execute", of_type=str
             ),
@@ -89,6 +96,19 @@ class ExecutionPlanSnapshot(
             ),
         )
 
+    @property
+    def asset_selection(self) -> AbstractSet[AssetKey]:
+        asset_keys = set()
+
+        for step in self.steps:
+            if step.key in self.step_keys_to_execute:
+                for output in step.outputs:
+                    asset_key = check.not_none(output.properties).asset_key
+                    if asset_key:
+                        asset_keys.add(asset_key)
+
+        return asset_keys
+
     @property
     def step_deps(self):
         # Construct dependency dictionary (downstream to upstreams)
@@ -111,13 +131,15 @@ class ExecutionPlanSnapshotErrorData(
     NamedTuple("_ExecutionPlanSnapshotErrorData", [("error", Optional[SerializableErrorInfo])])
 ):
     def __new__(cls, error: Optional[SerializableErrorInfo]):
-        return super(ExecutionPlanSnapshotErrorData, cls).__new__(
+        return super().__new__(
             cls,
             error=check.opt_inst_param(error, "error", SerializableErrorInfo),
         )
 
 
-@whitelist_for_serdes(storage_field_names={"node_handle_id": "solid_handle_id"})
+@whitelist_for_serdes(
+    storage_field_names={"node_handle_id": "solid_handle_id"}, skip_when_none_fields={"pool"}
+)
 class ExecutionStepSnap(
     NamedTuple(
         "_ExecutionStepSnap",
@@ -130,6 +152,7 @@ class ExecutionStepSnap(
             ("metadata_items", Sequence["ExecutionPlanMetadataItemSnap"]),
             ("tags", Optional[Mapping[str, str]]),
             ("step_handle", Optional[StepHandleUnion]),
+            ("pool", Optional[str]),
         ],
     )
 ):
@@ -143,8 +166,9 @@ class ExecutionStepSnap(
         metadata_items: Sequence["ExecutionPlanMetadataItemSnap"],
         tags: Optional[Mapping[str, str]] = None,
         step_handle: Optional[StepHandleUnion] = None,
+        pool: Optional[str] = None,
     ):
-        return super(ExecutionStepSnap, cls).__new__(
+        return super().__new__(
             cls,
             key=check.str_param(key, "key"),
             inputs=check.sequence_param(inputs, "inputs", ExecutionStepInputSnap),
@@ -156,8 +180,26 @@ class ExecutionStepSnap(
             ),
             tags=check.opt_nullable_mapping_param(tags, "tags", key_type=str, value_type=str),
             step_handle=check.opt_inst_param(step_handle, "step_handle", StepHandleTypes),
+            # stores the pool arg as separate from the concurrency_key property since the
+            # snapshot may have been generated before concurrency_key was added as a separate
+            # argument
+            pool=check.opt_str_param(pool, "pool"),
         )
 
+    @property
+    def required_entity_keys(self) -> AbstractSet[EntityKey]:
+        """The set of entity keys on required outputs for this step."""
+        return {
+            output.entity_key
+            for output in self.outputs
+            if output.entity_key and output.properties and output.properties.is_required
+        }
+
+    @property
+    def entity_keys(self) -> AbstractSet[EntityKey]:
+        """The set of entity keys on all outputs for this step."""
+        return {output.entity_key for output in self.outputs if output.entity_key}
+
 
 @whitelist_for_serdes
 class ExecutionStepInputSnap(
@@ -178,7 +220,7 @@ class ExecutionStepInputSnap(
         upstream_output_handles: Sequence[StepOutputHandle],
         source: Optional[StepInputSourceUnion] = None,
     ):
-        return super(ExecutionStepInputSnap, cls).__new__(
+        return super().__new__(
             cls,
             check.str_param(name, "name"),
             check.str_param(dagster_type_key, "dagster_type_key"),
@@ -212,7 +254,7 @@ class ExecutionStepOutputSnap(
         node_handle: Optional[NodeHandle] = None,
         properties: Optional[StepOutputProperties] = None,
     ):
-        return super(ExecutionStepOutputSnap, cls).__new__(
+        return super().__new__(
             cls,
             check.str_param(name, "name"),
             check.str_param(dagster_type_key, "dagster_type_key"),
@@ -220,13 +262,19 @@ class ExecutionStepOutputSnap(
             check.opt_inst_param(properties, "properties", StepOutputProperties),
         )
 
+    @property
+    def entity_key(self) -> Optional[EntityKey]:
+        if self.properties:
+            return self.properties.asset_key or self.properties.asset_check_key
+        return None
+
 
 @whitelist_for_serdes
 class ExecutionPlanMetadataItemSnap(
     NamedTuple("_ExecutionPlanMetadataItemSnap", [("key", str), ("value", str)])
 ):
     def __new__(cls, key: str, value: str):
-        return super(ExecutionPlanMetadataItemSnap, cls).__new__(
+        return super().__new__(
             cls,
             check.str_param(key, "key"),
             check.str_param(value, "value"),
@@ -275,19 +323,22 @@ def _snapshot_from_execution_step(execution_step: IExecutionStep) -> ExecutionSt
             list(map(_snapshot_from_step_output, execution_step.step_outputs)),
             key=lambda so: so.name,
         ),
-        node_handle_id=execution_step.node_handle.to_string(),
+        node_handle_id=str(execution_step.node_handle),
         kind=execution_step.kind,
-        metadata_items=sorted(
-            [
-                ExecutionPlanMetadataItemSnap(key=key, value=value)
-                for key, value in execution_step.tags.items()
-            ],
-            key=lambda md: md.key,
-        )
-        if execution_step.tags
-        else [],
+        metadata_items=(
+            sorted(
+                [
+                    ExecutionPlanMetadataItemSnap(key=key, value=value)
+                    for key, value in execution_step.tags.items()
+                ],
+                key=lambda md: md.key,
+            )
+            if execution_step.tags
+            else []
+        ),
         tags=execution_step.tags,
         step_handle=execution_step.handle,
+        pool=execution_step.pool,
     )
 
 
@@ -302,7 +353,7 @@ def snapshot_from_execution_plan(
             list(map(_snapshot_from_execution_step, execution_plan.steps)), key=lambda es: es.key
         ),
         artifacts_persisted=execution_plan.artifacts_persisted,
-        pipeline_snapshot_id=pipeline_snapshot_id,
+        job_snapshot_id=pipeline_snapshot_id,
         step_keys_to_execute=execution_plan.step_keys_to_execute,
         initial_known_state=execution_plan.known_state,
         snapshot_version=CURRENT_SNAPSHOT_VERSION,
