@@ -5,6 +5,7 @@ Revises: c39c047fa021
 Create Date: 2020-06-10 10:00:57.793622
 
 """
+
 from alembic import op
 from dagster._core.storage.migration.utils import get_currently_upgrading_instance, has_table
 from sqlalchemy import inspect
@@ -26,7 +27,7 @@ def upgrade():
         return
     instance = get_currently_upgrading_instance()
     if instance.scheduler:
-        instance.scheduler.wipe(instance)
+        instance.scheduler.wipe(instance)  # pyright: ignore[reportAttributeAccessIssue]
 
     #   No longer dropping the "schedules" table here, since
     #   the 0.10.0 migration checks for the presence of the "schedules"
