@@ -3,24 +3,25 @@ import os
 import shutil
 import uuid
 from abc import ABC, abstractmethod
+from collections.abc import Iterator
 from contextlib import contextmanager
-from typing import BinaryIO, ContextManager, Iterator, Optional, TextIO, Union
+from typing import BinaryIO, ContextManager, Optional, TextIO, Union  # noqa: UP035
 
 from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import public
 from dagster._config import Field, StringSource
-from dagster._core.definitions.resource_definition import resource
+from dagster._core.definitions.resource_definition import dagster_maintained_resource, resource
 from dagster._core.execution.context.init import InitResourceContext
 from dagster._core.instance import DagsterInstance
+from dagster._core.storage.temp_file_manager import TempfileManager
 from dagster._utils import mkdir_p
 
-from .temp_file_manager import TempfileManager
-
 IOStream: TypeAlias = Union[TextIO, BinaryIO]
 
 
+@public
 class FileHandle(ABC):
     """A reference to a file as manipulated by a FileManager.
 
@@ -41,6 +42,7 @@ class FileHandle(ABC):
         raise NotImplementedError()
 
 
+@public
 class LocalFileHandle(FileHandle):
     """A reference to a file on a local filesystem."""
 
@@ -60,6 +62,7 @@ class LocalFileHandle(FileHandle):
         return self._path
 
 
+@public
 class FileManager(ABC):
     """Base class for all file managers in dagster.
 
@@ -167,6 +170,7 @@ class FileManager(ABC):
         raise NotImplementedError()
 
 
+@dagster_maintained_resource
 @resource(config_schema={"base_dir": Field(StringSource, is_required=False)})
 def local_file_manager(init_context: InitResourceContext) -> "LocalFileManager":
     """FileManager that provides abstract access to a local filesystem.
@@ -223,7 +227,8 @@ def local_file_manager(init_context: InitResourceContext) -> "LocalFileManager":
     """
     return LocalFileManager(
         base_dir=init_context.resource_config.get(
-            "base_dir", os.path.join(init_context.instance.storage_directory(), "file_manager")  # type: ignore  # (possible none)
+            "base_dir",
+            os.path.join(init_context.instance.storage_directory(), "file_manager"),  # type: ignore  # (possible none)
         )
     )
 
@@ -255,13 +260,13 @@ class LocalFileManager(FileManager):
         check.inst_param(file_handle, "file_handle", FileHandle)
         with self.read(file_handle, "rb") as handle_obj:  # type: ignore  # (??)
             temp_file_obj = self._temp_file_manager.tempfile()
-            temp_file_obj.write(handle_obj.read())
+            temp_file_obj.write(handle_obj.read())  # pyright: ignore[reportCallIssue,reportArgumentType]
             temp_name = temp_file_obj.name
             temp_file_obj.close()
             return temp_name
 
     @contextmanager
-    def read(self, file_handle: LocalFileHandle, mode: str = "rb") -> Iterator[IOStream]:
+    def read(self, file_handle: LocalFileHandle, mode: str = "rb") -> Iterator[IOStream]:  # pyright: ignore[reportIncompatibleMethodOverride]
         check.inst_param(file_handle, "file_handle", LocalFileHandle)
         check.str_param(mode, "mode")
         check.param_invariant(mode in {"r", "rb"}, "mode")
@@ -270,7 +275,7 @@ class LocalFileManager(FileManager):
         with open(file_handle.path, mode, encoding=encoding) as file_obj:
             yield file_obj  # type: ignore  # (??)
 
-    def read_data(self, file_handle: LocalFileHandle) -> bytes:
+    def read_data(self, file_handle: LocalFileHandle) -> bytes:  # pyright: ignore[reportIncompatibleMethodOverride]
         with self.read(file_handle, mode="rb") as file_obj:
             return file_obj.read()  # type: ignore  # (??)
 
