@@ -1,45 +1,48 @@
 from abc import abstractmethod
 from collections import defaultdict
+from collections.abc import Mapping, Sequence
 from datetime import datetime
-from typing import (
-    Any,
-    Callable,
-    ContextManager,
-    Mapping,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Type,
-    TypeVar,
-)
+from typing import Any, Callable, ContextManager, NamedTuple, Optional, TypeVar  # noqa: UP035
 
-import pendulum
 import sqlalchemy as db
 import sqlalchemy.exc as db_exc
+from dagster_shared.serdes import deserialize_value
 from sqlalchemy.engine import Connection
 
 import dagster._check as check
+from dagster._core.definitions.asset_key import EntityKey
+from dagster._core.definitions.declarative_automation.serialized_objects import (
+    AutomationConditionEvaluationWithRunIds,
+)
 from dagster._core.definitions.run_request import InstigatorType
 from dagster._core.errors import DagsterInvariantViolationError
 from dagster._core.scheduler.instigation import (
+    AutoMaterializeAssetEvaluationRecord,
     InstigatorState,
+    InstigatorStatus,
     InstigatorTick,
     TickData,
     TickStatus,
 )
-from dagster._core.storage.sql import SqlAlchemyQuery, SqlAlchemyRow
-from dagster._serdes import serialize_value
-from dagster._serdes.serdes import deserialize_value
-from dagster._utils import PrintFn, utc_datetime_from_timestamp
-
-from .base import ScheduleStorage
-from .migration import (
+from dagster._core.storage.schedules.base import ScheduleStorage
+from dagster._core.storage.schedules.migration import (
     OPTIONAL_SCHEDULE_DATA_MIGRATIONS,
     REQUIRED_SCHEDULE_DATA_MIGRATIONS,
     SCHEDULE_JOBS_SELECTOR_ID,
     SCHEDULE_TICKS_SELECTOR_ID,
 )
-from .schema import InstigatorsTable, JobTable, JobTickTable, SecondaryIndexMigrationTable
+from dagster._core.storage.schedules.schema import (
+    AssetDaemonAssetEvaluationsTable,
+    InstigatorsTable,
+    JobTable,
+    JobTickTable,
+    SecondaryIndexMigrationTable,
+)
+from dagster._core.storage.sql import SqlAlchemyQuery, SqlAlchemyRow
+from dagster._core.storage.sqlalchemy_compat import db_fetch_mappings, db_select, db_subquery
+from dagster._serdes import serialize_value
+from dagster._time import datetime_from_timestamp, get_current_datetime
+from dagster._utils import PrintFn
 
 T_NamedTuple = TypeVar("T_NamedTuple", bound=NamedTuple)
 
@@ -60,7 +63,7 @@ class SqlScheduleStorage(ScheduleStorage):
         return res
 
     def _deserialize_rows(
-        self, rows: Sequence[SqlAlchemyRow], as_type: Type[T_NamedTuple]
+        self, rows: Sequence[SqlAlchemyRow], as_type: type[T_NamedTuple]
     ) -> Sequence[T_NamedTuple]:
         return list(map(lambda r: deserialize_value(r[0], as_type), rows))
 
@@ -69,24 +72,33 @@ class SqlScheduleStorage(ScheduleStorage):
         repository_origin_id: Optional[str] = None,
         repository_selector_id: Optional[str] = None,
         instigator_type: Optional[InstigatorType] = None,
+        instigator_statuses: Optional[set[InstigatorStatus]] = None,
     ) -> Sequence[InstigatorState]:
         check.opt_inst_param(instigator_type, "instigator_type", InstigatorType)
 
         if self.has_instigators_table() and self.has_built_index(SCHEDULE_JOBS_SELECTOR_ID):
-            query = db.select([InstigatorsTable.c.instigator_body]).select_from(InstigatorsTable)
+            query = db_select([InstigatorsTable.c.instigator_body]).select_from(InstigatorsTable)
             if repository_selector_id:
                 query = query.where(
                     InstigatorsTable.c.repository_selector_id == repository_selector_id
                 )
             if instigator_type:
                 query = query.where(InstigatorsTable.c.instigator_type == instigator_type.value)
+            if instigator_statuses:
+                query = query.where(
+                    InstigatorsTable.c.status.in_([status.value for status in instigator_statuses])
+                )
+
         else:
-            query = db.select([JobTable.c.job_body]).select_from(JobTable)
+            query = db_select([JobTable.c.job_body]).select_from(JobTable)
             if repository_origin_id:
                 query = query.where(JobTable.c.repository_origin_id == repository_origin_id)
             if instigator_type:
                 query = query.where(JobTable.c.job_type == instigator_type.value)
-
+            if instigator_statuses:
+                query = query.where(
+                    JobTable.c.status.in_([status.value for status in instigator_statuses])
+                )
         rows = self.execute(query)
         return self._deserialize_rows(rows, InstigatorState)
 
@@ -96,13 +108,13 @@ class SqlScheduleStorage(ScheduleStorage):
 
         if self.has_instigators_table() and self.has_built_index(SCHEDULE_JOBS_SELECTOR_ID):
             query = (
-                db.select([InstigatorsTable.c.instigator_body])
+                db_select([InstigatorsTable.c.instigator_body])
                 .select_from(InstigatorsTable)
                 .where(InstigatorsTable.c.selector_id == selector_id)
             )
         else:
             query = (
-                db.select([JobTable.c.job_body])
+                db_select([JobTable.c.job_body])
                 .select_from(JobTable)
                 .where(JobTable.c.job_origin_id == origin_id)
             )
@@ -114,7 +126,7 @@ class SqlScheduleStorage(ScheduleStorage):
         check.str_param(selector_id, "selector_id")
 
         query = (
-            db.select([JobTable.c.job_body])
+            db_select([JobTable.c.job_body])
             .select_from(JobTable)
             .where(JobTable.c.selector_id == selector_id)
         )
@@ -142,7 +154,7 @@ class SqlScheduleStorage(ScheduleStorage):
                     status=state.status.value,
                     instigator_type=state.instigator_type.value,
                     instigator_body=serialize_value(state),
-                    update_timestamp=pendulum.now("UTC"),
+                    update_timestamp=get_current_datetime(),
                 )
             )
 
@@ -174,15 +186,13 @@ class SqlScheduleStorage(ScheduleStorage):
         check.inst_param(state, "state", InstigatorState)
         if not self.get_instigator_state(state.instigator_origin_id, state.selector_id):
             raise DagsterInvariantViolationError(
-                "InstigatorState {id} is not present in storage".format(
-                    id=state.instigator_origin_id
-                )
+                f"InstigatorState {state.instigator_origin_id} is not present in storage"
             )
 
         values = {
             "status": state.status.value,
             "job_body": serialize_value(state),
-            "update_timestamp": pendulum.now("UTC"),
+            "update_timestamp": get_current_datetime(),
         }
         if self.has_instigators_table():
             values["selector_id"] = state.selector_id
@@ -220,7 +230,7 @@ class SqlScheduleStorage(ScheduleStorage):
 
     def _jobs_has_selector_state(self, conn: Connection, selector_id: str) -> bool:
         query = (
-            db.select([db.func.count()])
+            db_select([db.func.count()])
             .select_from(JobTable)
             .where(JobTable.c.selector_id == selector_id)
         )
@@ -243,9 +253,9 @@ class SqlScheduleStorage(ScheduleStorage):
         check.opt_list_param(statuses, "statuses", of_type=TickStatus)
 
         if before:
-            query = query.where(JobTickTable.c.timestamp < utc_datetime_from_timestamp(before))
+            query = query.where(JobTickTable.c.timestamp < datetime_from_timestamp(before))
         if after:
-            query = query.where(JobTickTable.c.timestamp > utc_datetime_from_timestamp(after))
+            query = query.where(JobTickTable.c.timestamp > datetime_from_timestamp(after))
         if limit:
             query = query.limit(limit)
         if statuses:
@@ -264,6 +274,10 @@ class SqlScheduleStorage(ScheduleStorage):
         table_names = db.inspect(conn).get_table_names()
         return "instigators" in table_names
 
+    def _has_asset_daemon_asset_evaluations_table(self, conn: Connection) -> bool:
+        table_names = db.inspect(conn).get_table_names()
+        return "asset_daemon_asset_evaluations" in table_names
+
     def get_batch_ticks(
         self,
         selector_ids: Sequence[str],
@@ -282,8 +296,8 @@ class SqlScheduleStorage(ScheduleStorage):
             )
             .label("rank")
         )
-        subquery = (
-            db.select(
+        subquery = db_subquery(
+            db_select(
                 [
                     JobTickTable.c.id,
                     JobTickTable.c.selector_id,
@@ -293,7 +307,6 @@ class SqlScheduleStorage(ScheduleStorage):
             )
             .select_from(JobTickTable)
             .where(JobTickTable.c.selector_id.in_(selector_ids))
-            .alias("subquery")
         )
         if statuses:
             subquery = subquery.where(
@@ -301,7 +314,7 @@ class SqlScheduleStorage(ScheduleStorage):
             )
 
         query = (
-            db.select([subquery.c.id, subquery.c.selector_id, subquery.c.tick_body])
+            db_select([subquery.c.id, subquery.c.selector_id, subquery.c.tick_body])
             .order_by(subquery.c.rank.asc())
             .where(subquery.c.rank <= limit)
         )
@@ -315,6 +328,24 @@ class SqlScheduleStorage(ScheduleStorage):
             results[selector_id].append(InstigatorTick(tick_id, tick_data))
         return results
 
+    def get_tick(self, tick_id: int) -> InstigatorTick:
+        check.int_param(tick_id, "tick_id")
+
+        query = (
+            db_select([JobTickTable.c.id, JobTickTable.c.tick_body])
+            .select_from(JobTickTable)
+            .where(JobTickTable.c.id == tick_id)
+        )
+
+        rows = self.execute(query)
+        if not rows:
+            raise DagsterInvariantViolationError(
+                f"InstigatorTick {tick_id} is not present in storage"
+            )
+
+        tick_id, tick_data = rows[0]
+        return InstigatorTick(tick_id, deserialize_value(tick_data, TickData))
+
     def get_ticks(
         self,
         origin_id: str,
@@ -331,7 +362,7 @@ class SqlScheduleStorage(ScheduleStorage):
         check.opt_list_param(statuses, "statuses", of_type=TickStatus)
 
         base_query = (
-            db.select([JobTickTable.c.id, JobTickTable.c.tick_body])
+            db_select([JobTickTable.c.id, JobTickTable.c.tick_body])
             .select_from(JobTickTable)
             .order_by(JobTickTable.c.timestamp.desc())
         )
@@ -362,7 +393,7 @@ class SqlScheduleStorage(ScheduleStorage):
             "job_origin_id": tick_data.instigator_origin_id,
             "status": tick_data.status.value,
             "type": tick_data.instigator_type.value,
-            "timestamp": utc_datetime_from_timestamp(tick_data.timestamp),
+            "timestamp": datetime_from_timestamp(tick_data.timestamp),
             "tick_body": serialize_value(tick_data),
         }
         if self.has_instigators_table() and tick_data.selector_id:
@@ -386,7 +417,7 @@ class SqlScheduleStorage(ScheduleStorage):
         values = {
             "status": tick.status.value,
             "type": tick.instigator_type.value,
-            "timestamp": utc_datetime_from_timestamp(tick.timestamp),
+            "timestamp": datetime_from_timestamp(tick.timestamp),
             "tick_body": serialize_value(tick.tick_data),
         }
         if self.has_instigators_table() and tick.selector_id:
@@ -410,7 +441,7 @@ class SqlScheduleStorage(ScheduleStorage):
         check.float_param(before, "before")
         check.opt_list_param(tick_statuses, "tick_statuses", of_type=TickStatus)
 
-        utc_before = utc_datetime_from_timestamp(before)
+        utc_before = datetime_from_timestamp(before)
 
         query = JobTickTable.delete().where(JobTickTable.c.timestamp < utc_before)
         if tick_statuses:
@@ -434,6 +465,101 @@ class SqlScheduleStorage(ScheduleStorage):
         with self.connect() as conn:
             conn.execute(query)
 
+    @property
+    def supports_auto_materialize_asset_evaluations(self) -> bool:
+        with self.connect() as conn:
+            return self._has_asset_daemon_asset_evaluations_table(conn)
+
+    def add_auto_materialize_asset_evaluations(
+        self,
+        evaluation_id: int,
+        asset_evaluations: Sequence[AutomationConditionEvaluationWithRunIds[EntityKey]],
+    ):
+        if not asset_evaluations:
+            return
+
+        with self.connect() as conn:
+            for evaluation in asset_evaluations:
+                insert_stmt = AssetDaemonAssetEvaluationsTable.insert().values(
+                    [
+                        {
+                            "evaluation_id": evaluation_id,
+                            "asset_key": evaluation.key.to_db_string(),
+                            "asset_evaluation_body": serialize_value(evaluation),
+                            "num_requested": evaluation.num_requested,
+                        }
+                    ]
+                )
+                try:
+                    conn.execute(insert_stmt)
+                except db_exc.IntegrityError:
+                    conn.execute(
+                        AssetDaemonAssetEvaluationsTable.update()
+                        .where(
+                            db.and_(
+                                AssetDaemonAssetEvaluationsTable.c.evaluation_id == evaluation_id,
+                                AssetDaemonAssetEvaluationsTable.c.asset_key
+                                == evaluation.key.to_db_string(),
+                            )
+                        )
+                        .values(
+                            asset_evaluation_body=serialize_value(evaluation),
+                            num_requested=evaluation.num_requested,
+                        )
+                    )
+
+    def get_auto_materialize_asset_evaluations(
+        self, key: EntityKey, limit: int, cursor: Optional[int] = None
+    ) -> Sequence[AutoMaterializeAssetEvaluationRecord]:
+        with self.connect() as conn:
+            query = (
+                db_select(
+                    [
+                        AssetDaemonAssetEvaluationsTable.c.id,
+                        AssetDaemonAssetEvaluationsTable.c.asset_evaluation_body,
+                        AssetDaemonAssetEvaluationsTable.c.evaluation_id,
+                        AssetDaemonAssetEvaluationsTable.c.create_timestamp,
+                        AssetDaemonAssetEvaluationsTable.c.asset_key,
+                    ]
+                )
+                .where(AssetDaemonAssetEvaluationsTable.c.asset_key == key.to_db_string())
+                .order_by(AssetDaemonAssetEvaluationsTable.c.evaluation_id.desc())
+            ).limit(limit)
+
+            if cursor is not None:
+                query = query.where(AssetDaemonAssetEvaluationsTable.c.evaluation_id < cursor)
+
+            rows = db_fetch_mappings(conn, query)
+            return [AutoMaterializeAssetEvaluationRecord.from_db_row(row) for row in rows]
+
+    def get_auto_materialize_evaluations_for_evaluation_id(
+        self, evaluation_id: int
+    ) -> Sequence[AutoMaterializeAssetEvaluationRecord]:
+        with self.connect() as conn:
+            query = db_select(
+                [
+                    AssetDaemonAssetEvaluationsTable.c.id,
+                    AssetDaemonAssetEvaluationsTable.c.asset_evaluation_body,
+                    AssetDaemonAssetEvaluationsTable.c.evaluation_id,
+                    AssetDaemonAssetEvaluationsTable.c.create_timestamp,
+                    AssetDaemonAssetEvaluationsTable.c.asset_key,
+                ]
+            ).where(AssetDaemonAssetEvaluationsTable.c.evaluation_id == evaluation_id)
+
+            rows = db_fetch_mappings(conn, query)
+            return [AutoMaterializeAssetEvaluationRecord.from_db_row(row) for row in rows]
+
+    def purge_asset_evaluations(self, before: float):
+        check.float_param(before, "before")
+
+        utc_before = datetime_from_timestamp(before)
+        query = AssetDaemonAssetEvaluationsTable.delete().where(
+            AssetDaemonAssetEvaluationsTable.c.create_timestamp < utc_before
+        )
+
+        with self.connect() as conn:
+            conn.execute(query)
+
     def wipe(self) -> None:
         """Clears the schedule storage."""
         with self.connect() as conn:
@@ -442,6 +568,8 @@ class SqlScheduleStorage(ScheduleStorage):
             conn.execute(JobTickTable.delete())
             if self._has_instigators_table(conn):
                 conn.execute(InstigatorsTable.delete())
+            if self._has_asset_daemon_asset_evaluations_table(conn):
+                conn.execute(AssetDaemonAssetEvaluationsTable.delete())
 
     # MIGRATIONS
 
@@ -454,7 +582,7 @@ class SqlScheduleStorage(ScheduleStorage):
             return False
 
         query = (
-            db.select([1])
+            db_select([1])
             .where(SecondaryIndexMigrationTable.c.name == migration_name)
             .where(SecondaryIndexMigrationTable.c.migration_completed != None)  # noqa: E711
             .limit(1)
