@@ -1,16 +1,15 @@
 import os
 import sys
+from contextlib import ExitStack
 from typing import Optional
 
 import click
+from dagster_shared.cli import WorkspaceOpts, workspace_options
+from dagster_shared.ipc import interrupt_on_ipc_shutdown_message
 
 from dagster import __version__ as dagster_version
-from dagster._cli.workspace.cli_target import (
-    ClickArgMapping,
-    ClickArgValue,
-    get_workspace_load_target,
-    workspace_target_argument,
-)
+from dagster._cli.utils import assert_no_remaining_opts, get_instance_for_cli
+from dagster._cli.workspace.cli_target import workspace_opts_to_load_target
 from dagster._core.instance import DagsterInstance, InstanceRef
 from dagster._core.telemetry import telemetry_wrapper
 from dagster._daemon.controller import (
@@ -23,7 +22,7 @@ from dagster._daemon.controller import (
 )
 from dagster._daemon.daemon import get_telemetry_daemon_session_id
 from dagster._serdes import deserialize_value
-from dagster._utils.interrupts import capture_interrupts
+from dagster._utils.interrupts import capture_interrupts, setup_interrupt_handlers
 
 
 def _get_heartbeat_tolerance():
@@ -42,9 +41,23 @@ def _get_heartbeat_tolerance():
     help="Set the log level for any code servers spun up by the daemon.",
     show_default=True,
     default="warning",
-    type=click.Choice(
-        ["critical", "error", "warning", "info", "debug", "trace"], case_sensitive=False
-    ),
+    type=click.Choice(["critical", "error", "warning", "info", "debug"], case_sensitive=False),
+)
+@click.option(
+    "--log-level",
+    help="Set the log level for any code servers spun up by the daemon.",
+    show_default=True,
+    default="info",
+    type=click.Choice(["critical", "error", "warning", "info", "debug"], case_sensitive=False),
+    envvar="DAGSTER_DAEMON_LOG_LEVEL",
+)
+@click.option(
+    "--log-format",
+    type=click.Choice(["colored", "json", "rich"], case_sensitive=False),
+    show_default=True,
+    required=False,
+    default="colored",
+    help="Format of the log output from the webserver",
 )
 @click.option(
     "--instance-ref",
@@ -52,33 +65,58 @@ def _get_heartbeat_tolerance():
     required=False,
     hidden=True,
 )
-@workspace_target_argument
+@click.option(
+    "--shutdown-pipe",
+    type=click.INT,
+    required=False,
+    hidden=True,
+    help="Internal use only. Pass a readable pipe file descriptor to the daemon process that will be monitored for a shutdown signal.",
+)
+@workspace_options
 def run_command(
     code_server_log_level: str,
+    log_level: str,
+    log_format: str,
     instance_ref: Optional[str],
-    **kwargs: ClickArgValue,
+    shutdown_pipe: Optional[int],
+    **other_opts: object,
 ) -> None:
+    workspace_opts = WorkspaceOpts.extract_from_cli_options(other_opts)
+    assert_no_remaining_opts(other_opts)
+
+    # Essential on windows-- will set up windows interrupt signals to raise KeyboardInterrupt
+    setup_interrupt_handlers()
+
     try:
-        with capture_interrupts():
-            with DagsterInstance.from_ref(
-                deserialize_value(instance_ref, InstanceRef)
-            ) if instance_ref else DagsterInstance.get() as instance:
-                _daemon_run_command(instance, code_server_log_level, kwargs)
+        with ExitStack() as stack:
+            if shutdown_pipe:
+                stack.enter_context(interrupt_on_ipc_shutdown_message(shutdown_pipe))
+            stack.enter_context(capture_interrupts())
+            with get_instance_for_cli(
+                instance_ref=deserialize_value(instance_ref, InstanceRef) if instance_ref else None
+            ) as instance:
+                _daemon_run_command(
+                    instance, log_level, code_server_log_level, log_format, workspace_opts
+                )
     except KeyboardInterrupt:
         return  # Exit cleanly on interrupt
 
 
 @telemetry_wrapper(metadata={"DAEMON_SESSION_ID": get_telemetry_daemon_session_id()})
 def _daemon_run_command(
-    instance: DagsterInstance, code_server_log_level: str, kwargs: ClickArgMapping
+    instance: DagsterInstance,
+    log_level: str,
+    code_server_log_level: str,
+    log_format: str,
+    workspace_opts: WorkspaceOpts,
 ) -> None:
-    workspace_load_target = get_workspace_load_target(kwargs)
-
     with daemon_controller_from_instance(
         instance,
-        workspace_load_target=workspace_load_target,
+        workspace_load_target=workspace_opts_to_load_target(workspace_opts),
         heartbeat_tolerance_seconds=_get_heartbeat_tolerance(),
+        log_level=log_level,
         code_server_log_level=code_server_log_level,
+        log_format=log_format,
     ) as controller:
         controller.check_daemon_loop()
 
@@ -88,7 +126,7 @@ def _daemon_run_command(
     help="Check for recent heartbeats from the daemon.",
 )
 def liveness_check_command() -> None:
-    with DagsterInstance.get() as instance:
+    with get_instance_for_cli() as instance:
         if all_daemons_live(instance, heartbeat_tolerance_seconds=_get_heartbeat_tolerance()):
             click.echo("Daemon live")
         else:
@@ -101,7 +139,7 @@ def liveness_check_command() -> None:
     help="Wipe all heartbeats from storage.",
 )
 def wipe_command() -> None:
-    with DagsterInstance.get() as instance:
+    with get_instance_for_cli() as instance:
         instance.wipe_daemon_heartbeats()
         click.echo("Daemon heartbeats wiped")
 
@@ -111,7 +149,7 @@ def wipe_command() -> None:
     help="Read and write a heartbeat",
 )
 def debug_heartbeat_command() -> None:
-    with DagsterInstance.get() as instance:
+    with get_instance_for_cli() as instance:
         debug_daemon_heartbeats(instance)
 
 
@@ -120,7 +158,7 @@ def debug_heartbeat_command() -> None:
     help="Log all heartbeat statuses",
 )
 def debug_heartbeat_dump_command() -> None:
-    with DagsterInstance.get() as instance:
+    with get_instance_for_cli() as instance:
         for daemon_status in get_daemon_statuses(instance, instance.get_required_daemon_types()):
             click.echo(daemon_status)
 
@@ -152,4 +190,4 @@ cli = create_dagster_daemon_cli()
 
 
 def main() -> None:
-    cli(obj={})  # pylint:disable=E1123
+    cli(obj={})
