@@ -1,29 +1,37 @@
 import importlib
 from abc import ABC, abstractmethod
-from typing import Any, Dict, Mapping, NamedTuple, Optional, Type, TypeVar, Union, overload
+from collections.abc import Mapping
+from typing import TYPE_CHECKING, Any, NamedTuple, Optional, TypeVar, Union, cast, overload
 
+import dagster_shared.check as check
+from dagster_shared.serdes.serdes import NamedTupleSerializer, whitelist_for_serdes
+from dagster_shared.yaml_utils import load_run_config_yaml
 from typing_extensions import Self
 
-import dagster._check as check
-from dagster._config.config_schema import UserConfigSchema
+from dagster._annotations import public
 from dagster._utils import convert_dagster_submodule_name
-from dagster._utils.yaml_utils import load_run_config_yaml
 
-from .serdes import (
-    NamedTupleSerializer,
-    whitelist_for_serdes,
-)
+if TYPE_CHECKING:
+    from dagster._config.config_schema import UserConfigSchema
 
+# This should have a bound of ConfigurableClass, but the type checker has difficulty with putting
+# the ConfigurableClass interface on our storage classes. The concrete implementations of these
+# classes end up implementing the ConfigurableClass interface without inheriting from it, so we
+# don't actually bound this var.
 T_ConfigurableClass = TypeVar("T_ConfigurableClass")
 
 
 class ConfigurableClassDataSerializer(NamedTupleSerializer["ConfigurableClassData"]):
-    def after_pack(self, **packed: Any) -> Dict[str, Any]:
-        packed["module_name"] = convert_dagster_submodule_name(packed["module_name"], "public")
-        return packed
+    def pack_items(self, *args, **kwargs):
+        for k, v in super().pack_items(*args, **kwargs):
+            if k == "module_name":
+                yield k, convert_dagster_submodule_name(v, "public")  # pyright: ignore[reportArgumentType]
+            else:
+                yield k, v
 
 
 @whitelist_for_serdes(serializer=ConfigurableClassDataSerializer)
+@public
 class ConfigurableClassData(
     NamedTuple(
         "_ConfigurableClassData",
@@ -44,7 +52,7 @@ class ConfigurableClassData(
     """
 
     def __new__(cls, module_name: str, class_name: str, config_yaml: str):
-        return super(ConfigurableClassData, cls).__new__(
+        return super().__new__(
             cls,
             convert_dagster_submodule_name(check.str_param(module_name, "module_name"), "private"),
             check.str_param(class_name, "class_name"),
@@ -63,15 +71,13 @@ class ConfigurableClassData(
         }
 
     @overload
-    def rehydrate(self, as_type: Type[T_ConfigurableClass]) -> T_ConfigurableClass:
-        ...
+    def rehydrate(self, as_type: None = ...) -> "ConfigurableClass": ...
 
     @overload
-    def rehydrate(self, as_type: None = ...) -> "ConfigurableClass":
-        ...
+    def rehydrate(self, as_type: type[T_ConfigurableClass]) -> T_ConfigurableClass: ...
 
     def rehydrate(
-        self, as_type: Optional[Type[T_ConfigurableClass]] = None
+        self, as_type: Optional[type[T_ConfigurableClass]] = None
     ) -> Union["ConfigurableClass", T_ConfigurableClass]:
         from dagster._config import process_config, resolve_to_config_type
         from dagster._core.errors import DagsterInvalidConfigError
@@ -84,7 +90,12 @@ class ConfigurableClassData(
                 f"configurable class {self.module_name}.{self.class_name}"
             )
         try:
-            klass = getattr(module, self.class_name)
+            # All rehydrated classes are expected to implement the ConfigurableClass interface and
+            # will error when we call `klass.from_config_value` and `klass.config_type` below if
+            # they do not. However, not all rehydrated classes actually have `ConfigurableClass` as
+            # an ancestor due to some subtleties around multiple abstract classes that cause an
+            # error when `ConfigurableClass` is added as an ancestor to storage classes.
+            klass = cast("type[ConfigurableClass]", getattr(module, self.class_name))
         except AttributeError:
             check.failed(
                 f"Couldn't find class {self.class_name} in module when attempting to load the "
@@ -109,6 +120,7 @@ class ConfigurableClassData(
         return klass.from_config_value(self, check.not_none(result.value))
 
 
+@public
 class ConfigurableClass(ABC):
     """Abstract mixin for classes that can be loaded from config.
 
@@ -193,7 +205,7 @@ class ConfigurableClass(ABC):
         """
 
 
-def class_from_code_pointer(module_name: str, class_name: str) -> Type[object]:
+def class_from_code_pointer(module_name: str, class_name: str) -> type[object]:
     try:
         module = importlib.import_module(module_name)
     except ModuleNotFoundError:
